"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generate = void 0;
const base_1 = require("@modular-service/base");
const path_1 = __importDefault(require("path"));
const migrations_1 = require("@modular-service/plugin-knex/dist/migrations");
const postgresql_1 = require("@testcontainers/postgresql");
const plugin_knex_1 = require("@modular-service/plugin-knex");
const knex_1 = __importDefault(require("knex"));
const kanel = __importStar(require("kanel"));
const kanel_zod_1 = require("kanel-zod");
const kanel_knex_1 = require("kanel-knex");
const recase_1 = require("@kristiandupont/recase");
function generate(relativeMigrationsDirectory, outputDirectory) {
    const dirname = (0, base_1.getRuntimeDirname)();
    (0, base_1.deferGenerator)(async () => {
        const postgresContainer = await new postgresql_1.PostgreSqlContainer().start();
        try {
            // # migrate db
            const db = (0, knex_1.default)({
                client: 'pg',
                connection: postgresContainer.getConnectionUri()
            });
            try {
                // check connection
                await db.raw('select 1');
                // run migrations
                const migrations = await (0, migrations_1.getMigrations)(path_1.default.join(dirname, relativeMigrationsDirectory));
                await plugin_knex_1.knexMigrator.migrate('up', db, path_1.default.join(dirname, relativeMigrationsDirectory), migrations);
            }
            finally {
                await db.destroy();
            }
            // # generate schema
            const outputPath = path_1.default.join(dirname, outputDirectory);
            await kanel.processDatabase({
                connection: postgresContainer.getConnectionUri(),
                outputPath,
                ...kanelConfig()
            });
        }
        finally {
            await postgresContainer.stop();
        }
        return {};
    });
}
exports.generate = generate;
function kanelConfig() {
    const toPascalCase = (0, recase_1.recase)('snake', 'pascal');
    return {
        preDeleteOutputFolder: true,
        resolveViews: true,
        preRenderHooks: [
            (0, kanel_zod_1.makeGenerateZodSchemas)({
                getZodSchemaMetadata: kanel_zod_1.defaultGetZodSchemaMetadata,
                getZodIdentifierMetadata: kanel_zod_1.defaultGetZodIdentifierMetadata,
                zodTypeMap: {
                    ...kanel_zod_1.defaultZodTypeMap,
                    'pg_catalog.uuid': 'z.string().uuid()'
                }
            }),
            kanel_knex_1.generateKnexTablesModule,
            // generateMigrationCheck
        ],
        typeFilter: kanel_knex_1.knexTypeFilter,
        // // Add a comment about the entity that the type represents above each type.
        // getMetadata: (details, generateFor) => {
        //   const { comment: strippedComment } = tryParse(details.comment);
        //   const isAgentNoun = ['initializer', 'mutator'].includes(generateFor ?? '');
        //   const relationComment = isAgentNoun
        //     ? `Represents the ${generateFor} for the ${details.kind} ${details.schemaName}.${details.name}`
        //     : `Represents the ${details.kind} ${details.schemaName}.${details.name}`;
        //   const suffix = isAgentNoun ? `_${generateFor}` : '';
        //   return {
        //     name: toPascalCase(details.name + suffix),
        //     comment: [relationComment, ...(strippedComment ? [strippedComment] : [])],
        //     path: path.join(outputPath, toPascalCase(details.name)),
        //   };
        // },
        // // Add a comment that says what the type of the column/attribute is in the database.
        // getPropertyMetadata: (property, _details, generateFor) => {
        //   const { comment: strippedComment } = tryParse(property.comment);
        //   return {
        //     name: property.name,
        //     comment: [
        //       `Database type: ${property.expandedType}`,
        //       ...(strippedComment ? [strippedComment] : []),
        //       ...(generateFor === 'initializer' && property.defaultValue
        //         ? [`Default value: ${property.defaultValue}`]
        //         : []),
        //     ]
        //   }
        // },
        // This implementation will generate flavored instead of branded types.
        // See: https://spin.atomicobject.com/2018/01/15/typescript-flexible-nominal-typing/
        generateIdentifierType: (c, d, config) => {
            const name = toPascalCase(d.name) + toPascalCase(c.name);
            const innerType = kanel.resolveType(c, d, {
                ...config,
                // Explicitly disable identifier resolution so we get the actual inner type here
                generateIdentifierType: undefined,
            });
            const imports = [];
            let type = innerType;
            if (typeof innerType === "object") {
                // Handle non-primitives
                type = innerType.name;
                imports.push(...innerType.typeImports);
            }
            return {
                declarationType: 'typeDeclaration',
                name,
                exportAs: 'named',
                typeDefinition: [`${type} & { __flavor?: '${name}' }`],
                typeImports: imports,
                comment: [`Identifier type for ${d.schemaName}.${d.name}`],
            };
        },
        // customTypeMap: {
        //   // A text search vector could be stored as a set of strings. See Film.ts for an example.
        //   'pg_catalog.tsvector': 'Set<string>',
        //   // The bytea package (https://www.npmjs.com/package/postgres-bytea) could be used for byte arrays.
        //   // See Staff.ts for an example.
        //   'pg_catalog.bytea': { name: 'bytea', typeImports: [{ name: 'bytea', path: 'bytea', isAbsolute: true, isDefault: true, importAsType: false }] },
        //   // Columns with the following types would probably just be strings in TypeScript.
        //   'pg_catalog.bpchar': 'string',
        //   'public.citext': 'string',
        //   'pg_catalog.float8': 'number'
        // },
    };
}
//# sourceMappingURL=index.js.map